@@ -30,6 +30,7 @@ const char copyright[] = "Copyright (c) 2006-2009 Roy Marples";
 #include <sys/file.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
+#include <sys/time.h>
 #include <sys/types.h>
 #include <sys/uio.h>
 
@@ -71,6 +72,11 @@ const char copyright[] = "Copyright (c) 2006-2009 Roy Marples";
 /* We should define a maximum for the NAK exponential backoff */ 
 #define NAKOFF_MAX              60
 
+/* Wait N nanoseconds between sending a RELEASE and dropping the address.
+ * This gives the kernel enough time to actually send it. */
+#define RELEASE_DELAY_S		0
+#define RELEASE_DELAY_NS	10000000
+
 int options = 0;
 int pidfd = -1;
 struct interface *ifaces = NULL;
@@ -136,7 +142,7 @@ read_pid(void)
 static void
 usage(void)
 {
-	printf("usage: "PACKAGE" [-dgknpqxyADEGHKLOTV] [-c script] [-f file]"
+	printf("usage: "PACKAGE" [-dgknpqwxyADEGHKLOTV] [-c script] [-f file]"
 	    " [-e var=val]\n"
 	    "              [-h hostname] [-i classID ] [-l leasetime]"
 	    " [-m metric] [-o option]\n"
@@ -184,11 +190,18 @@ cleanup(void)
 }
 
 /* ARGSUSED */
-_noreturn void
+void
 handle_exit_timeout(_unused void *arg)
 {
+	int timeout;
+
 	syslog(LOG_ERR, "timed out");
-	exit(EXIT_FAILURE);
+	if (!(options & DHCPCD_TIMEOUT_IPV4LL))
+		exit(EXIT_FAILURE);
+	options &= ~DHCPCD_TIMEOUT_IPV4LL;
+	timeout = (PROBE_NUM * PROBE_MAX) + PROBE_WAIT + 1;
+	syslog(LOG_WARNING, "allowing %d seconds for IPv4LL timeout", timeout);
+	add_timeout_sec(timeout, handle_exit_timeout, NULL);
 }
 
 void
@@ -386,12 +399,6 @@ start_expire(void *arg)
 		start_interface(iface);
 }
 
-void
-send_decline(struct interface *iface)
-{
-	send_message(iface, DHCP_DECLINE, NULL);
-}
-
 static void
 log_dhcp(int lvl, const char *msg,
     const struct interface *iface, const struct dhcp_message *dhcp)
@@ -516,10 +523,9 @@ handle_dhcp(struct interface *iface, struct dhcp_message **dhcpp)
 	{
 		lease->frominfo = 0;
 		lease->addr.s_addr = dhcp->yiaddr;
-		lease->server.s_addr = INADDR_ANY;
-		if (type != 0)
-			get_option_addr(&lease->server,
-			    dhcp, DHO_SERVERID);
+		if (type == 0 ||
+		    get_option_addr(&lease->server, dhcp, DHO_SERVERID) != 0)
+			lease->server.s_addr = INADDR_ANY;
 		log_dhcp(LOG_INFO, "offered", iface, dhcp);
 		free(state->offer);
 		state->offer = dhcp;
@@ -570,9 +576,10 @@ handle_dhcp(struct interface *iface, struct dhcp_message **dhcpp)
 		state->offer = dhcp;
 		*dhcpp = NULL;
 	}
-	lease->frominfo = 0;
 
+	lease->frominfo = 0;
 	delete_timeout(NULL, iface);
+
 	/* We now have an offer, so close the DHCP sockets.
 	 * This allows us to safely ARP when broken DHCP servers send an ACK
 	 * follows by an invalid NAK. */
@@ -702,18 +709,32 @@ open_sockets(struct interface *iface)
 static void
 send_release(struct interface *iface)
 {
+	struct timespec ts;
+
 	if (iface->state->lease.addr.s_addr &&
 	    !IN_LINKLOCAL(htonl(iface->state->lease.addr.s_addr)))
 	{
 		syslog(LOG_INFO, "%s: releasing lease of %s",
 		    iface->name, inet_ntoa(iface->state->lease.addr));
 		open_sockets(iface);
+		iface->state->xid = arc4random();
 		send_message(iface, DHCP_RELEASE, NULL);
+		/* Give the packet a chance to go before dropping the ip */
+		ts.tv_sec = RELEASE_DELAY_S;
+		ts.tv_nsec = RELEASE_DELAY_NS;
+		nanosleep(&ts, NULL);
 		drop_config(iface, "RELEASE");
 	}
 	unlink(iface->leasefile);
 }
 
+void
+send_decline(struct interface *iface)
+{
+	open_sockets(iface);
+	send_message(iface, DHCP_DECLINE, NULL);
+}
+
 static void
 configure_interface1(struct interface *iface)
 {
@@ -727,7 +748,7 @@ configure_interface1(struct interface *iface)
 	if (iface->flags & IFF_NOARP ||
 	    ifo->options & (DHCPCD_INFORM | DHCPCD_STATIC))
 		ifo->options &= ~(DHCPCD_ARP | DHCPCD_IPV4LL);
-	if (ifo->options & DHCPCD_LINK && carrier_status(iface->name) == -1)
+	if (ifo->options & DHCPCD_LINK && carrier_status(iface) == -1)
 		ifo->options &= ~DHCPCD_LINK;
 	
 	if (ifo->metric != -1)
@@ -772,6 +793,9 @@ configure_interface1(struct interface *iface)
 			    iface->hwlen);
 		}
 	}
+	if (ifo->options & DHCPCD_CLIENTID)
+		syslog(LOG_DEBUG, "%s: using ClientID %s", iface->name,
+		    hwaddr_ntoa(iface->clientid + 1, *iface->clientid));
 }
 
 int
@@ -820,6 +844,7 @@ static void
 handle_carrier(const char *ifname)
 {
 	struct interface *iface;
+	int carrier;
 
 	if (!(options & DHCPCD_LINK))
 		return;
@@ -828,11 +853,10 @@ handle_carrier(const char *ifname)
 			break;
 	if (!iface || !(iface->state->options->options & DHCPCD_LINK))
 		return;
-	switch (carrier_status(iface->name)) {
-	case -1:
-		syslog(LOG_ERR, "carrier_status: %m");
-		break;
-	case 0:
+	carrier = carrier_status(iface);
+	if (carrier == -1)
+		syslog(LOG_ERR, "%s: carrier_status: %m", ifname);
+	else if (carrier == 0 || !(iface->flags & IFF_RUNNING)) {
 		if (iface->carrier != LINK_DOWN) {
 			iface->carrier = LINK_DOWN;
 			syslog(LOG_INFO, "%s: carrier lost", iface->name);
@@ -840,8 +864,7 @@ handle_carrier(const char *ifname)
 			delete_timeouts(iface, start_expire, NULL);
 			drop_config(iface, "NOCARRIER");
 		}
-		break;
-	default:
+	} else if (carrier == 1 && (iface->flags & IFF_RUNNING)) {
 		if (iface->carrier != LINK_UP) {
 			iface->carrier = LINK_UP;
 			syslog(LOG_INFO, "%s: carrier acquired", iface->name);
@@ -853,7 +876,6 @@ handle_carrier(const char *ifname)
 			run_script(iface);
 			start_interface(iface);
 		}
-		break;
 	}
 }
 
@@ -908,7 +930,7 @@ start_rebind(void *arg)
 {
 	struct interface *iface = arg;
 
-	syslog(LOG_ERR, "%s: failed to renew, attmepting to rebind",
+	syslog(LOG_ERR, "%s: failed to renew, attempting to rebind",
 	    iface->name);
 	iface->state->state = DHS_REBIND;
 	delete_timeout(send_renew, iface);
@@ -1164,7 +1186,7 @@ init_state(struct interface *iface, int argc, char **argv)
 		run_script(iface);
 
 	if (ifs->options->options & DHCPCD_LINK) {
-		switch (carrier_status(iface->name)) {
+		switch (carrier_status(iface)) {
 		case 0:
 			iface->carrier = LINK_DOWN;
 			ifs->reason = "NOCARRIER";
@@ -1187,7 +1209,7 @@ void
 handle_interface(int action, const char *ifname)
 {
 	struct interface *ifs, *ifp, *ifn, *ifl = NULL;
-	const char * const argv[] = { "dhcpcd", ifname };
+	const char * const argv[] = { ifname }; 
 	int i;
 
 	if (action == -1) {
@@ -1209,23 +1231,30 @@ handle_interface(int action, const char *ifname)
 			return;
 	}
 
-	if ((ifs = discover_interfaces(2, UNCONST(argv)))) {
-		for (ifp = ifs; ifp; ifp = ifp->next) {
-			/* Check if we already have the interface */
-			for (ifn = ifaces; ifn; ifn = ifn->next) {
-				if (strcmp(ifn->name, ifp->name) == 0)
-					break;
-				ifl = ifn;
-			}
-			if (ifn)
-				continue;
-			init_state(ifp, 2, UNCONST(argv));
+	ifs = discover_interfaces(-1, UNCONST(argv));
+	for (ifp = ifs; ifp; ifp = ifp->next) {
+		if (strcmp(ifp->name, ifname) != 0)
+			continue;
+		/* Check if we already have the interface */
+		for (ifn = ifaces; ifn; ifn = ifn->next) {
+			if (strcmp(ifn->name, ifp->name) == 0)
+				break;
+			ifl = ifn;
+		}
+		if (ifn) {
+			/* The flags and hwaddr could have changed */
+			ifn->flags = ifp->flags;
+			ifn->hwlen = ifp->hwlen;
+			if (ifp->hwlen != 0)
+				memcpy(ifn->hwaddr, ifp->hwaddr, ifn->hwlen);
+		} else {
 			if (ifl)
 				ifl->next = ifp;
 			else
 				ifaces = ifp;
-			start_interface(ifp);
 		}
+		init_state(ifp, 0, NULL);
+		start_interface(ifp);
 	}
 }
 
@@ -1613,7 +1642,7 @@ main(int argc, char **argv)
 	if (options & DHCPCD_DEBUG)
 		setlogmask(LOG_UPTO(LOG_DEBUG));
 	else if (options & DHCPCD_QUIET)
-		setlogmask(LOG_UPTO(LOG_WARNING));
+		close(STDERR_FILENO);
 
 	if (!(options & DHCPCD_TEST)) {
 		/* If we have any other args, we should run as a single dhcpcd
@@ -1663,6 +1692,10 @@ main(int argc, char **argv)
 		if (pid == 0 || kill(pid, sig) != 0) {
 			if (sig != SIGALRM)
 				syslog(LOG_ERR, ""PACKAGE" not running");
+			if (pid != 0 && errno != ESRCH) {
+				syslog(LOG_ERR, "kill: %m");
+				exit(EXIT_FAILURE);
+			}
 			unlink(pidfile);
 			if (sig != SIGALRM)
 				exit(EXIT_FAILURE);
@@ -1734,7 +1767,7 @@ main(int argc, char **argv)
 		}
 	}
 
-	if (init_socket() == -1) {
+	if (init_sockets() == -1) {
 		syslog(LOG_ERR, "init_socket: %m");
 		exit(EXIT_FAILURE);
 	}
@@ -1748,17 +1781,11 @@ main(int argc, char **argv)
 
 	ifc = argc - optind;
 	ifv = argv + optind;
-	if (options & DHCPCD_BACKGROUND ||
-	    (ifc == 0 && options & DHCPCD_LINK && options & DHCPCD_DAEMONISE))
-	{
-		daemonise();
-	} else if (options & DHCPCD_DAEMONISE) {
-		oi = ifo->timeout;
-		if (ifo->options & DHCPCD_IPV4LL)
-			oi += 10;
-		add_timeout_sec(oi, handle_exit_timeout, NULL);
-	}
-	free_options(ifo);
+
+	/* When running dhcpcd against a single interface, we need to retain
+	 * the old behaviour of waiting for an IP address */
+	if (ifc == 1)
+		options |= DHCPCD_WAITIP;
 
 	ifaces = discover_interfaces(ifc, ifv);
 	for (i = 0; i < ifc; i++) {
@@ -1772,15 +1799,59 @@ main(int argc, char **argv)
 	if (!ifaces) {
 		if (ifc == 0)
 			syslog(LOG_ERR, "no valid interfaces found");
+		else
+			exit(EXIT_FAILURE);
 		if (!(options & DHCPCD_LINK)) {
-			syslog(LOG_ERR, "aborting as we're not backgrounding"
-			    " with link detection");
+			syslog(LOG_ERR,
+			    "aborting as link detection is disabled");
 			exit(EXIT_FAILURE);
 		}
 	}
 
-	for (iface = ifaces; iface; iface = iface->next)
+	if (options & DHCPCD_BACKGROUND)
+		daemonise();
+
+	opt = 0;
+	for (iface = ifaces; iface; iface = iface->next) {
 		init_state(iface, argc, argv);
+		if (iface->carrier != LINK_DOWN)
+			opt = 1;
+	}
+
+	if (!(options & DHCPCD_BACKGROUND)) {
+		/* If we don't have a carrier, we may have to wait for a second
+		 * before one becomes available if we brought an interface up. */
+		if (opt == 0 &&
+		    options & DHCPCD_LINK &&
+		    options & DHCPCD_WAITUP &&
+		    !(options & DHCPCD_WAITIP))
+		{
+			ts.tv_sec = 1;
+			ts.tv_nsec = 0;
+			nanosleep(&ts, NULL);
+			for (iface = ifaces; iface; iface = iface->next) {
+				handle_carrier(iface->name);
+				if (iface->carrier != LINK_DOWN) {
+					opt = 1;
+					break;
+				}
+			}
+		}
+		if (opt == 0 &&
+		    options & DHCPCD_LINK &&
+		    !(options & DHCPCD_WAITIP))
+		{
+			syslog(LOG_WARNING, "no interfaces have a carrier");
+			daemonise();
+		} else if (options & DHCPCD_DAEMONISE && ifo->timeout > 0) {
+			if (options & DHCPCD_IPV4LL)
+				options |= DHCPCD_TIMEOUT_IPV4LL;
+			add_timeout_sec(ifo->timeout, handle_exit_timeout, NULL);
+		}
+
+	}
+	free_options(ifo);
+
 	sort_interfaces();
 	for (iface = ifaces; iface; iface = iface->next)
 		add_timeout_sec(0, start_interface, iface);
