@@ -74,8 +74,11 @@ _open_link_socket(struct sockaddr_nl *nl)
 }
 
 int
-init_socket(void)
+init_sockets(void)
 {
+	if ((socket_afnet = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
+		return -1;
+	set_cloexec(socket_afnet);
 	sock_fd = _open_link_socket(&sock_nl);
 	set_cloexec(sock_fd);
 	return sock_fd;
@@ -174,6 +177,7 @@ link_route(struct nlmsghdr *nlm)
 	len = NLMSG_PAYLOAD(nlm, sizeof(*rtm));
 	rt.iface = NULL;
 	rt.dest.s_addr = INADDR_ANY;
+	rt.net.s_addr = INADDR_ANY;
 	rt.gate.s_addr = INADDR_ANY;
 	rt.next = NULL;
 	metric = 0;
@@ -200,8 +204,6 @@ link_route(struct nlmsghdr *nlm)
 	}
 	if (rt.iface != NULL) {
 		if (metric == rt.iface->metric) {
-			if (rt.dest.s_addr == INADDR_BROADCAST)
-				rt.dest.s_addr = INADDR_ANY;
 			inet_cidrtoaddr(rtm->rtm_dst_len, &rt.net);
 			route_deleted(&rt);
 		}
